@@ -11,7 +11,9 @@ let vec: Vec<i32> = vec![1, 2, 3];
 let arc_vec = Arc::new(vec);
 let pierce = Pierce::new(arc_vec);
 
-// Here, the execution jumps directly to the slice to call `.get(...)`. (Without Pierce it would have to jump to the Vec first, than from the Vec to the slice).
+// Here, the execution jumps directly to the slice to call `.get(...)`.
+// Without Pierce it would have to jump to the Vec first,
+// than from the Vec to the slice.
 pierce.get(0).unwrap();
 ```
 
@@ -30,7 +32,9 @@ then follow the inner pointer to where the underlying data is. Two [Deref]-ings.
 let vec: Vec<i32> = vec![1, 2, 3];
 let arc_vec = Arc::new(vec);
 
-// Here, the `Arc<Vec<i32>>` is first dereferenced to the `Vec<i32>`, then the Vec is dereferenced to the underlying i32 slice, on which `.get(...)` is called.
+// Here, the `Arc<Vec<i32>>` is first dereferenced to the `Vec<i32>`,
+// then the Vec is dereferenced to the underlying i32 slice,
+// on which `.get(...)` is called.
 arc_vec.get(0).unwrap();
 ```
 
@@ -185,13 +189,13 @@ assert_ne!(&*weird_pierce, first);
 
 ## Fallback
 
-For Pierce to function optimally, **the final deref target must not be inside the outer pointer**,
+For Pierce to function optimally, **the double-deref target must not be inside the outer pointer**,
 (it should be e.g. somehwere else on the heap or in the static region).
 
 This condition is met by most common smart pointers, including (but not limited to) [Box], [Vec], [String], [Arc][std::sync::Arc], [Rc][std::rc::Rc].
 
 For pointers that don't meet this condition,
-Pierce pin it to the heap using `Box` to give it a stable address,
+Pierce falls back to pin it to the heap using `Box` to give it a stable address,
 so that the cache would not be left dangling if the Pierce (and the outer pointer in it) is moved.
 
 You should avoid using Pierce if your doubly-nested pointer points to itself anyway.
@@ -208,7 +212,7 @@ where
     target: NonNull<<T::Target as Deref>::Target>,
 }
 
-pub enum PierceOuter<T>
+enum PierceOuter<T>
 where
     T: Deref,
     T::Target: Deref,
@@ -244,7 +248,7 @@ where
     Create a Pierce out of the given nested pointer.
     This method derefs T twice and cache the address where the inner pointer points to.
 
-    Deref-ing the create Pierce returns the cached reference directly. `deref` is not called on T.
+    Deref-ing the created Pierce returns the cached reference directly. `deref` is not called on T.
      */
     #[inline]
     pub fn new(outer: T) -> Self {
@@ -267,11 +271,32 @@ where
         }
     }
 
-    /** Borrow the outer pointer T
+    /** Create a new Pierce if it won't fall back to pinning
 
-    You can then call the methods on &T.
+    See the "Limitations" section at the [crate docs][crate] for more info about falling back.
 
-    You can even call `deref` twice on &T directly to bypass Pierce's cache:
+    This returns None if the double-deref target of the given `T` points to somewhere in T.
+    */
+    #[inline]
+    pub fn new_no_pin(outer: T) -> Option<Self> {
+        let inner: &T::Target = outer.deref();
+        let target: &<T::Target as Deref>::Target = inner.deref();
+        if needs_pinning(&outer, target) {
+            let target = NonNull::from(target);
+            Some(Self {
+                outer: PierceOuter::Normal(outer),
+                target,
+            })
+        } else {
+            None
+        }
+    }
+
+    /** Borrow the outer pointer `T`
+
+    You can then call the methods on `&T`.
+
+    You can even call `deref` twice on `&T` yourself to bypass Pierce's cache:
     ```
     # use pierce::Pierce;
     use std::ops::Deref;
@@ -303,16 +328,14 @@ where
         }
     }
 
-    /** Whether or not Pierce has cached the target
+    /** Whether or not Pierce has falled back to pinning to the heap
 
-    Pierce only cache the target when it is safe to do so. See the "Limitations" section at the [crate docs][crate].
-
-    This method returns true if the target is cached, false if Pierce is falling back to double-derefing every time.
+    See the "Limitations" section at the [crate docs][crate] for more info about falling back.
     */
-    pub fn is_cached(&self) -> bool {
+    pub fn is_fallback(&self) -> bool {
         match self.outer {
-            PierceOuter::Normal(..) => true,
-            PierceOuter::Fallback(..) => false,
+            PierceOuter::Normal(..) => false,
+            PierceOuter::Fallback(..) => true,
         }
     }
 }
@@ -415,7 +438,7 @@ mod tests {
         let p2 = p1.clone();
         p1.get(0).unwrap().borrow_mut().add_assign(5);
         assert_eq!(*p2.get(0).unwrap().borrow(), 6);
-        assert_eq!(p1.is_cached(), true);
+        assert_eq!(p1.is_fallback(), false);
     }
 
     #[test]
@@ -426,7 +449,7 @@ mod tests {
         let a = Rc::new(v);
         let pierce = Pierce::new(a);
         assert_eq!(&*pierce, "hello world");
-        assert_eq!(pierce.is_cached(), true);
+        assert_eq!(pierce.is_fallback(), false);
     }
 
     #[test]
@@ -435,7 +458,7 @@ mod tests {
         let a = Box::new(v);
         let pierce = Pierce::new(a);
         assert_eq!(*pierce.get(2).unwrap(), 3);
-        assert_eq!(pierce.is_cached(), true);
+        assert_eq!(pierce.is_fallback(), false);
     }
 
     #[test]
@@ -445,7 +468,7 @@ mod tests {
         assert_eq!(*Box::deref(Pierce::deref(&pierce_once)), 42);
         let pierce_twice = Pierce::new(pierce_once);
         assert_eq!(*Pierce::deref(&pierce_twice), 42);
-        assert_eq!(pierce_twice.is_cached(), true);
+        assert_eq!(pierce_twice.is_fallback(), false);
     }
 
     #[test]
@@ -474,7 +497,7 @@ mod tests {
         let weird_pierce = Pierce::new(Box::new(WeirdPointer {
             inner: RefCell::new(true),
         }));
-        assert_eq!(weird_pierce.is_cached(), true);
+        assert_eq!(weird_pierce.is_fallback(), false);
         assert_eq!(&**weird_normal, "hello");
         assert_eq!(&*weird_pierce, "hello");
         assert_eq!(&**weird_normal, "world");
@@ -495,7 +518,7 @@ mod tests {
         let c = StackPtr(b);
         let p = Pierce::new(c);
 
-        assert_eq!(p.is_cached(), false);
+        assert_eq!(p.is_fallback(), true);
     }
     #[test]
     fn test_box_stack() {
@@ -504,7 +527,7 @@ mod tests {
         let c = Box::new(b);
         let p = Pierce::new(c);
 
-        assert_eq!(p.is_cached(), true);
+        assert_eq!(p.is_fallback(), false);
     }
     #[test]
     fn test_stack_box() {
@@ -513,7 +536,7 @@ mod tests {
         let c = StackPtr(b);
         let p = Pierce::new(c);
 
-        assert_eq!(p.is_cached(), true);
+        assert_eq!(p.is_fallback(), false);
     }
 
     #[test]
@@ -523,6 +546,18 @@ mod tests {
         let c = Box::new(b);
         let p = Pierce::new(c);
 
-        assert_eq!(p.is_cached(), true);
+        assert_eq!(p.is_fallback(), false);
+    }
+
+    #[test]
+    fn test_no_pin_some() {
+        let b = Box::new(Box::new(50));
+        assert!(Pierce::new_no_pin(b).is_some());
+    }
+
+    #[test]
+    fn test_no_pin_none() {
+        let b = StackPtr(StackPtr(6));
+        assert!(Pierce::new_no_pin(b).is_none());
     }
 }
